%undefine _ld_as_needed

Summary: Serial Graph Partitioning and Fill-reducing Matrix Ordering
Name:    metis
Version: 5.1.0
Release: 2%{?dist}
License: ASL 2.0 and BSD and LGPLv2+
URL:     https://github.com/KarypisLab/METIS
Source0: https://src.fedoraproject.org/lookaside/pkgs/metis/%{name}-%{version}.tar.gz/5465e67079419a69e0116de24fce58fe/%{name}-%{version}.tar.gz
Patch3000: %{name}-libmetis.patch
Patch3001: %{name}-shared-GKlib.patch
Patch3002: %{name}-width-datatype.patch
Patch3003: %{name}-GKREGEX-GKRAND-LIBSUFFIX-fix.patch
Patch3004: %{name}_lib64.patch
Patch3005: %{name}-pcre2.patch

BuildRequires: make cmake3 gcc gcc-c++ pcre2-devel help2man chrpath

%description
METIS is a set of serial programs for partitioning graphs, partitioning finite element meshes, and
producing fill reducing orderings for sparse matrices.  The algorithms implemented in METIS are
based on the multilevel recursive-bisection, multilevel k-way, and multi-constraint partitioning
schemes developed in our lab.  METIS is distributed with OpenMP support.

%package devel
Summary: METIS headers and development-related files
Requires: %{name} = %{version}-%{release}

%description devel
Header and library files of Metis.

%package -n metis64
Summary: Serial Graph Partitioning and Fill-reducing Matrix Ordering (64bit INTEGER)

%description -n metis64
METIS is a set of serial programs for partitioning graphs, partitioning finite element meshes, and
producing fill reducing orderings for sparse matrices.  The algorithms implemented in METIS are
based on the multilevel recursive-bisection, multilevel k-way, and multi-constraint partitioning
schemes developed in our lab.  METIS is distributed with OpenMP support.  This build has 64bit
INTEGER support.

%package -n metis64-devel
Summary: METIS development libraries (64bit INTEGER)
Requires: metis64 = %{version}-%{release}

%description -n metis64-devel
Header and library files of Metis, OpenMP version (64bit INTEGER).

%prep
%setup -qc
 
pushd %{name}-%{version}
%patch3000 -p0
%patch3001 -p0
%patch3003 -p0
%patch3005 -p0

sed -e 's|-O3||g' -i GKlib/GKlibSystem.cmake
popd

cp -a %{name}-%{version} metis64

%build
PCRE_LDFLAGS="-lpcre2-posix"
%cmake3 -S %{name}-%{version} -B %{name}-%{version}/build \
 -DCMAKE_BUILD_TYPE:STRING=Release \
 -DGKLIB_PATH=%{name}-%{version}/GKlib  \
 -DSHARED:BOOL=TRUE \
 -DOPENMP:BOOL=ON \
 -DPCRE:BOOL=ON \
 -DCMAKE_C_FLAGS:STRING="%{optflags} -pthread" \
 -DCMAKE_SHARED_LINKER_FLAGS_RELEASE:STRING="%{__global_ldflags} $PCRE_LDFLAGS" \
 -DCMAKE_EXE_LINKER_FLAGS_RELEASE:STRING="%{__global_ldflags} $PCRE_LDFLAGS" \
 -DCMAKE_VERBOSE_MAKEFILE:BOOL=TRUE \
 -DCMAKE_INSTALL_PREFIX:PATH=%{_prefix}
%make_build -C %{name}-%{version}/build

cd metis64
patch -p0 < %{PATCH3002}
patch -p0 < %{PATCH3004}
cd ..
mkdir -p metis64/build64
PCRE_LDFLAGS="-lpcre2-posix"
%cmake3 -S metis64 -B metis64/build64 \
 -DCMAKE_BUILD_TYPE:STRING=Release \
 -DGKLIB_PATH=metis64/GKlib  \
 -DSHARED:BOOL=TRUE \
 -DOPENMP:BOOL=ON \
 -DPCRE:BOOL=ON \
 -DCMAKE_C_FLAGS:STRING="%{optflags} -pthread" \
 -DCMAKE_SHARED_LINKER_FLAGS_RELEASE:STRING="%{__global_ldflags} $PCRE_LDFLAGS" \
 -DCMAKE_EXE_LINKER_FLAGS_RELEASE:STRING="%{__global_ldflags} $PCRE_LDFLAGS" \
 -DCMAKE_VERBOSE_MAKEFILE:BOOL=TRUE \
 -DCMAKE_INSTALL_PREFIX:PATH=%{_prefix} ..
%make_build -C metis64/build64

%install
pushd %{name}-%{version}/build
%make_install

LD_PRELOAD=%{buildroot}%{_libdir}/lib%{name}.so.0 \
help2man --version-string="%{version}" -n "Partitions a graph into a specified number of parts." \
 -N --output="gpmetis.1" --no-discard-stderr --help-option="-help" %{buildroot}%{_bindir}/gpmetis

LD_PRELOAD=%{buildroot}%{_libdir}/lib%{name}.so.0 \
help2man --version-string="%{version}" \
 -n "Computes a fill-reducing ordering of the vertices of the graph using multilevel nested dissection." \
 -N --output="ndmetis.1" --no-discard-stderr --help-option="-help" %{buildroot}%{_bindir}/ndmetis

LD_PRELOAD=%{buildroot}%{_libdir}/lib%{name}.so.0 \
help2man --version-string="%{version}" -n "Partitions a mesh into a specified number of parts." \
 -N --output="mpmetis.1" --no-discard-stderr --help-option="-help" %{buildroot}%{_bindir}/mpmetis

LD_PRELOAD=%{buildroot}%{_libdir}/lib%{name}.so.0 \
help2man --version-string="%{version}" -n "Converts a mesh into a graph that is compatible with METIS." \
 -N --output="m2gmetis.1" --no-discard-stderr -h "-help" %{buildroot}%{_bindir}/m2gmetis

mkdir -p %{buildroot}%{_mandir}/man1
mv *.1 %{buildroot}%{_mandir}/man1
popd

mv %{buildroot}%{_includedir}/metis.h %{buildroot}%{_includedir}/metis32.h

pushd metis64/build64
%make_install
mv %{buildroot}%{_includedir}/metis.h %{buildroot}%{_includedir}/metis64.h
popd

mv %{buildroot}%{_includedir}/metis32.h %{buildroot}%{_includedir}/metis.h

chrpath -d %{buildroot}%{_bindir}/*

%check
cp -p %{buildroot}%{_bindir}/* %{name}-%{version}/graphs
pushd %{name}-%{version}/graphs
LD_LIBRARY_PATH=%{buildroot}%{_libdir}:$LD_LIBRARY_PATH ./ndmetis mdual.graph
LD_LIBRARY_PATH=%{buildroot}%{_libdir}:$LD_LIBRARY_PATH ./mpmetis metis.mesh 2
LD_LIBRARY_PATH=%{buildroot}%{_libdir}:$LD_LIBRARY_PATH ./gpmetis test.mgraph 4
LD_LIBRARY_PATH=%{buildroot}%{_libdir}:$LD_LIBRARY_PATH ./gpmetis copter2.graph 4
LD_LIBRARY_PATH=%{buildroot}%{_libdir}:$LD_LIBRARY_PATH ./graphchk 4elt.graph
LD_LIBRARY_PATH=%{buildroot}%{_libdir}:$LD_LIBRARY_PATH ./ndmetis64 mdual.graph
LD_LIBRARY_PATH=%{buildroot}%{_libdir}:$LD_LIBRARY_PATH ./mpmetis64 metis.mesh 2
LD_LIBRARY_PATH=%{buildroot}%{_libdir}:$LD_LIBRARY_PATH ./gpmetis64 test.mgraph 4
LD_LIBRARY_PATH=%{buildroot}%{_libdir}:$LD_LIBRARY_PATH ./gpmetis64 copter2.graph 4
LD_LIBRARY_PATH=%{buildroot}%{_libdir}:$LD_LIBRARY_PATH ./graphchk64 4elt.graph
popd

%files
%license %{name}-%{version}/LICENSE.txt
%doc %{name}-%{version}/Changelog %{name}-%{version}/manual/manual.pdf
%{_bindir}/cmpfillin
%{_bindir}/gpmetis
%{_bindir}/graphchk
%{_bindir}/m2gmetis
%{_bindir}/mpmetis
%{_bindir}/ndmetis
%{_libdir}/lib%{name}.so.*
%{_mandir}/man1/*.1.gz

%files devel
%{_includedir}/%{name}.h
%{_libdir}/lib%{name}.so

%files -n metis64
%license metis64/LICENSE.txt
%doc metis64/Changelog metis64/manual/manual.pdf
%{_bindir}/cmpfillin64
%{_bindir}/gpmetis64
%{_bindir}/graphchk64
%{_bindir}/m2gmetis64
%{_bindir}/mpmetis64
%{_bindir}/ndmetis64
%{_libdir}/lib%{name}64.so.*

%files -n metis64-devel
%{_includedir}/%{name}64.h
%{_libdir}/lib%{name}64.so

%changelog
* Fri Apr 28 2023 OpenCloudOS Release Engineering <releng@opencloudos.tech> - 5.1.0-2
- Rebuilt for OpenCloudOS Stream 23.05

* Wed Apr 26 2023 Shuo Wang <abushwang@tencent.com> - 5.1.0-1
- initial build
